#include "widget.h"

Display *dpy;

//QueryWindow
class QueryWindow : public NSButtonListener, public NSFrame {
public:
  QueryWindow();
  
  bool single() { return _single; }
  bool multiple() { return _multiple; }

  void action(const XEvent&, void*);

private:
  NSLabel label;
  NSButton singleButton;
  NSButton multipleButton;

  NSHContainer con;
  NSVContainer mainCon;

  bool _single;
  bool _multiple;
};

QueryWindow::QueryWindow()
  : label("How many do you want to move?"), singleButton(" Single ", this), multipleButton("Multiple", this),
    con(300, 100), mainCon(300, 150)
{
  con.add(&singleButton); con.add(&multipleButton);
  con.reallocate();
  
  mainCon.add(&label); mainCon.add(&con);
  mainCon.reallocate();

  container(&mainCon);

  _single = true;
  _multiple = false;
}

void QueryWindow::action(const XEvent& ev, void*)
{
  if (ev.xany.window == singleButton.window()) {
    _single = true;
    _multiple = false;
  } else if (ev.xany.window == multipleButton.window()) {
    _single = false;
    _multiple = true;
  }
}

//Score
class ScoreWindow : public NSFrame, public NSButtonListener {
public:
  ScoreWindow();
  
  void incWins();
  void incDefeats();

  void waitForEvent();

  void action(const XEvent&, void*);
    
private:
  void readScore();
  void writeScore() const;
  void setLabels();

  int _wins, _defeats;
  int _continuousWins;
  bool _previousWon;

  string saveFile;

  bool exitPressed;

  NSButton exitButton;
  NSLabel winLabel, defeatLabel, winpLabel, defeatpLabel, cwinLabel;
  NSVContainer mainCon;
};    

ScoreWindow::ScoreWindow()
    : exitButton("exit", this), winLabel(""), defeatLabel(""), winpLabel(""), defeatpLabel(""),
      cwinLabel(""), mainCon(300, 300)
{
    mainCon.add(&winLabel); mainCon.add(&defeatLabel);
    mainCon.add(&winpLabel); mainCon.add(&defeatpLabel);
    mainCon.add(&cwinLabel); mainCon.add(&exitButton);
    mainCon.reallocate();
    
    container(&mainCon);
    
    _wins = _defeats = 0;
    _continuousWins = 0;
    _previousWon = false;
    exitPressed = false;

    char* home = getenv("HOME");

    if (home == NULL) {
	fprintf(stderr, "Cannot get $HOME. Assuming I am at home directory now.\n");
	saveFile = ".freecell";
    } else {
	saveFile = home;
	saveFile += "/.freecell";
    }
    readScore();
    //    setLabels();
}

void ScoreWindow::incWins()
{
  if (_previousWon == false) 
    _previousWon = true;

  _continuousWins++;

  _wins++;
  writeScore();
}

void ScoreWindow::incDefeats()
{
  _previousWon = false;
  _continuousWins = 0;
  _defeats++;
  writeScore();
}

void ScoreWindow::readScore()
{
  FILE* fp = fopen(saveFile.c_str(), "r");
  const int lineLength = 30;
  char line[lineLength];

  if (fp == NULL) {
    writeScore();
    return;
  }

  if (fgets(line, lineLength, fp) == NULL) goto ERROR;
  if (sscanf(line, "Wins = %d", &_wins) != 1) goto ERROR;
  if (fgets(line, lineLength, fp) == NULL) goto ERROR;
  if (sscanf(line, "Defeats = %d", &_defeats) != 1) goto ERROR;
  if (fgets(line, lineLength, fp) == NULL) goto ERROR;
  if (sscanf(line, "Continuous Wins = %d", &_continuousWins) != 1) goto ERROR;
  if (fgets(line, lineLength, fp) == NULL) goto ERROR;
  int tmp; // To make compiler happy.
  if (sscanf(line, "Previous Won = %d", &tmp) != 1) goto ERROR;
  if (tmp == 0) _previousWon = false; else _previousWon = true;

  fclose(fp);
  return;

 ERROR:
  fprintf(stderr, "Error encountered while reading %s\n", saveFile.c_str());
  fclose(fp);
}

void ScoreWindow::writeScore() const
{
  FILE* fp = fopen(saveFile.c_str(), "w+");

  if (fp == NULL) {
    fprintf(stderr, "Cannot open %s\n", saveFile.c_str());
    return;
  }

  fprintf(fp, "Wins = %d\n", _wins);
  fprintf(fp, "Defeats = %d\n", _defeats);
  fprintf(fp, "Continuous Wins = %d\n", _continuousWins);
  fprintf(fp, "Previous Won = %d\n", _previousWon);

  fclose(fp);
  return;
}    

/*
static void printGeometry(Window w)
{
  Window root;
  int x, y;
  unsigned int width, height, border, depth;

  XGetGeometry(dpy, w, &root, &x, &y, &width, &height, &border, &depth);

  printf("x: %d y: %d "
	 "width: %d height: %d\n",
	 x, y, width, height);
}
*/

void ScoreWindow::waitForEvent()
{
  exitPressed = false;
  XFlush(dpy);
  setLabels();

  map();

  XEvent ev;
  NSWindow* win;
  while (!exitPressed) {
    XNextEvent(dpy, &ev);
    win = NSWindow::windowToNSWindow(ev.xany.window);
    if (win != 0) win->dispatchEvent(ev);
  }
  unmap();
}

void ScoreWindow::action(const XEvent&, void*)
{
    exitPressed = true;
}

void ScoreWindow::setLabels()
{
  static char winsStr[10], defeatsStr[10];
  static char winPStr[30], defeatPStr[30];
  static char cwStr[30];

  sprintf(winsStr, "Wins: %d", _wins);
  winLabel.label(winsStr);
  sprintf(defeatsStr, "Defeats: %d", _defeats);
  defeatLabel.label(defeatsStr);
  sprintf(winPStr, "Win Percentage: %2.0f%%",
	  _wins + _defeats == 0 ? 0 : ((float) _wins) / (_wins + _defeats) * 100);
  winpLabel.label(winPStr);
  sprintf(defeatPStr, "Defeat Percentage: %2.0f%%",
	  _wins + _defeats == 0 ? 0 : ((float) _defeats) / (_wins + _defeats) * 100);
  defeatpLabel.label(defeatPStr);
  sprintf(cwStr, "Countinuous Wins: %d", _continuousWins);
  cwinLabel.label(cwStr);

  mainCon.reallocate();
}    

int main()
{
  NSInitialize();
  
  dpy = NSdisplay();
/*
  QueryWindow q;
  
  q.map();
*/
  ScoreWindow s;
  
  s.waitForEvent();

  return 0;
}
